﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Web;
using Bricksoft;
using Bricksoft.IM.AIM;

namespace Bricksoft.IM.AIM.Demo
{
    public partial class frmRobot : Form
    {
        private AIMClient im = new AIMClient();
        private TreeNode rootNode = null;

        public delegate void MyInvoke(string str);

        public void showLog(string log)
        {
            if (txtLog.InvokeRequired)
            {
                MyInvoke _myInvoke = new MyInvoke(showLog);
                this.Invoke(_myInvoke, new object[] { log });
            }
            else
            {
                txtLog.Text = "[" + System.DateTime.Now.ToLocalTime() + "] " + log + "\r\n" + txtLog.Text;
            }
        }

        public frmRobot()
        {
            InitializeComponent();
          
            cbStatus.Items.Add("Available");
            cbStatus.Items.Add("Away");
            cbStatus.Items.Add("Invisible");

            im.OnError += delegate(object sender, string log) { this.Invoke(new OnLogEventHandler(OnLog), new object[] { sender, log }); };
            im.OnLog += delegate(object sender, string log) { this.Invoke(new OnLogEventHandler(OnLog), new object[] { sender, log }); };
            im.OnLogin += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnLogin)); };
            im.OnLogout += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnLogout)); };
            im.OnReceiveBuddyList += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnReceiveBuddyList)); };
            im.OnReceiveGroupList += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnReceiveGroupList)); };
            im.OnBuddyStatusChanged += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnBuddyStatusChanged), new object[] { sender, buddy }); };
            im.OnMessage += delegate(object sender, string buddy, string message) { this.Invoke(new OnMessageEventHandler(OnMessage), new object[] { sender, buddy, message }); };
            im.OnOfflineMessage += delegate(object sender, string buddy, string message, DateTime receivedDateTime) { this.Invoke(new OnOfflineMessageEventHandler(OnOfflineMessage), new object[] { sender, buddy, message, receivedDateTime }); };
            im.OnStatusChanged += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnStatusChanged), new object[] { sender, e }); };
            im.OnDeleteBuddy += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnDeleteBuddy), new object[] { sender, buddy }); };
            im.OnAddBuddy += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnAddBuddy), new object[] { sender, buddy }); };
        }

        private void OnBuddyAddYouRequest(object sender, string buddy, string requestMessage, ref bool accept)
        {
            if (MessageBox.Show("\""+ buddy + "\" added you. Do you want to \"" + buddy + "\" to see you?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                accept = true;
            }
            else
                accept = false;
        }

        private void OnAddGroup(object sender, string group)
        {
            TreeNode gn = new TreeNode(group);
            rootNode.Nodes.Add(gn);
            gn.ImageKey = "group.png";
            gn.SelectedImageKey = gn.ImageKey;
        }

        private void OnBuddyTyping(object sender, string buddy)
        {
            showLog(buddy + " is typing...");
        }

        private void OnAddBuddy(object sender, string buddy)
        {
            AIMBuddy ab = (AIMBuddy)im.BuddyList[buddy];

            TreeNode gn = FindNode(rootNode, ab.GroupID);

            TreeNode bn = new TreeNode(ab.Account);
            bn.ImageKey = statusAsImageKey(ab.Status);
            bn.SelectedImageKey = bn.ImageKey;
            bn.Tag = ab;
            ab.Tag = bn;

            if (gn == null)
                rootNode.Nodes.Add(bn);
            else
                gn.Nodes.Add(bn);
        }

        private void OnDeleteBuddy(object sender, string buddy)
        {
            AIMBuddy b = (AIMBuddy)im.BuddyList[buddy];
            (b.Tag as TreeNode).Remove();
        }

        private void OnStatusChanged(object sender, EventArgs e)
        {
            rootNode.ImageKey = statusAsImageKey(im.Status);
            rootNode.SelectedImageKey = rootNode.ImageKey;

            /*
            cbStatus.Items.Add("Available");
            cbStatus.Items.Add("Away");
            cbStatus.Items.Add("Invisible");
             */
            switch (im.Status)
            {
                case AIMStatus.aimAway :
                    cbStatus.SelectedIndex = 1;
                    break;

                case AIMStatus.aimInvisible:
                case AIMStatus.aimOffline:
                    cbStatus.SelectedIndex = 2;
                    break;
                case AIMStatus.aimOnline:
                    cbStatus.SelectedIndex = 0;
                    break;
            }
        }

        private string statusAsImageKey(AIMStatus s)
        {
            switch (s)
            {
                case AIMStatus.aimOnline:
                    return "available.png";
                case AIMStatus.aimInvisible:
                case AIMStatus.aimOffline:
                    return "offline.png";
                case AIMStatus.aimAway:
                    return "away.png";
                default:
                    return "";
            }
        }

        private void OnOfflineMessage(object sender, string buddy, string message, DateTime receivedDateTime)
        {
            showLog(buddy + " said(" + receivedDateTime.ToLocalTime() + "): " + message);
        }

        private void OnMessage(object sender, string buddy, string message)
        {
            showLog(buddy + " says: " + message);            
        }

        private void OnBuddyStatusChanged(object sender, string buddy)
        {
            AIMBuddy b = (AIMBuddy)im.BuddyList[buddy];
            ((TreeNode)b.Tag).ImageKey = statusAsImageKey(b.Status);
            ((TreeNode)b.Tag).SelectedImageKey = ((TreeNode)b.Tag).ImageKey;
        }

        private void OnLogin(object sender, EventArgs e)
        {
            rootNode = new TreeNode(im.Account);
            rootNode.ImageKey = statusAsImageKey(im.Status);
            rootNode.SelectedImageKey = rootNode.ImageKey;

            cmdLogin.Enabled = false;
            cbStatus.Enabled = true;

            tvIM.Nodes.Add(rootNode);

            rootNode.Text = im.Account;

            tabControl1.SelectedIndex = 1;

            lblStatus.Text = "Login";
        }

        private void OnLogout(object sender, EventArgs e)
        {
            tvIM.Nodes.Clear();

            cmdLogin.Enabled = true;
            cbStatus.Enabled = false;

            tabControl1.SelectedIndex = 0;

            lblStatus.Text = "Logout.";
        }

        private void OnLog(object sender, string log)
        {
            showLog(log);
            lblStatus.Text = log;
        }

        private TreeNode FindNode(TreeNode tnParent, int v)
        {

            if (tnParent == null) return null;

            if ((tnParent.ImageKey == "group.png") && ((((OSCARGroup)tnParent.Tag).GroupID == v)))
                return tnParent;

            TreeNode tnRet = null;

            foreach (TreeNode tn in tnParent.Nodes)
            {
                tnRet = FindNode(tn, v);
                if (tnRet != null) break;
            }
            return tnRet;
        }

        private void OnReceiveBuddyList(object sender, EventArgs e)
        {
            foreach (DictionaryEntry buddy in im.BuddyList)
            {
                AIMBuddy b = (AIMBuddy)buddy.Value;

                TreeNode gn = FindNode(rootNode, b.GroupID);

                TreeNode bn = new TreeNode(b.Account);
                bn.ImageKey = statusAsImageKey(b.Status);
                bn.SelectedImageKey = bn.ImageKey;
                bn.Tag = b;
                b.Tag = bn;

                if (gn == null)
                    rootNode.Nodes.Add(bn);
                else
                    gn.Nodes.Add(bn);
            }
            rootNode.ExpandAll();
        }

        private void OnReceiveGroupList(object sender, EventArgs e)
        {
            foreach (DictionaryEntry group in im.GroupList)
            {
                OSCARGroup g = (OSCARGroup)group.Value;

                TreeNode gn = new TreeNode(g.GroupName);
                rootNode.Nodes.Add(gn);
                gn.ImageKey = "group.png";
                gn.SelectedImageKey = gn.ImageKey;
                gn.Tag = g;
                g.Tag = gn;
            }

            rootNode.ExpandAll();
        }

        private void cmdLogin_Click(object sender, EventArgs e)
        {
            im.Account = txtEmail.Text;
            im.Password = txtPassword.Text;
            im.Login();
        }

        private void cbStatus_SelectedIndexChanged(object sender, EventArgs e)
        {
            /*
            cbStatus.Items.Add("Available");
            cbStatus.Items.Add("Away");
            cbStatus.Items.Add("Invisible");
             */
            switch (cbStatus.SelectedIndex)
            {
                case 0:
                    im.Status = AIMStatus.aimOnline;
                    break;
                case 1:
                    im.Status = AIMStatus.aimAway;
                    break;
                case 2:
                    im.Status = AIMStatus.aimInvisible;
                    break;
            }
        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void addBuddyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //add buddy
            /*if (im.Logined)
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.setCaption("Add Buddy:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                    im.AddBuddy(frmInput.MessageStr);
            }*/
        }

        private void logoutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            im.Logout();
        }


        private void sendMessageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if ((tvIM.SelectedNode != null)&&(tvIM.SelectedNode.Tag is AIMBuddy))
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.setCaption("Instant Message:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                {
                    im.SendMessage((tvIM.SelectedNode.Tag as AIMBuddy).Account, frmInput.MessageStr);
                }
            }
        }

        private void snedOfflineMessageToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void loginToolStripMenuItem_Click(object sender, EventArgs e)
        {
            cmdLogin_Click(sender,e);
        }

        private void sendShakeToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void pbAvatar_MouseDoubleClick(object sender, MouseEventArgs e)
        {
        }

        private void toolStripMenuItem4_Click(object sender, EventArgs e)
        {
        }

        private void button1_Click(object sender, EventArgs e)
        {
        }

        private void button1_Click_1(object sender, EventArgs e)
        {
        }

        private void frmRobot_Shown(object sender, EventArgs e)
        {
        }

        private void frmRobot_Activated(object sender, EventArgs e)
        {
        }

        private void menuStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {

        }

        private void toolStripMenuItem4_Click_1(object sender, EventArgs e)
        {

        }

        private void toolStripMenuItem5_Click(object sender, EventArgs e)
        {
        }

        private void toolStripMenuItem6_Click(object sender, EventArgs e)
        {

        }

    }
}