using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Bricksoft;
using Bricksoft.IM.Yahoo;

namespace Bricksoft.IM.YahooCF.Demo
{
    public partial class frmYahooCF : Form
    {
        public delegate void MyInvoke(string str);
        private TreeNode rootNode;
        private YahooClient yahoo = new YahooClient();

        public void showLog(string log)
        {
            if (txtLog.InvokeRequired)
            {
                MyInvoke _myInvoke = new MyInvoke(showLog);
                this.Invoke(_myInvoke, new object[] { log });
            }
            else
            {
                txtLog.Text = "[" + System.DateTime.Now.ToLocalTime() + "] " + log + "\r\n" + txtLog.Text;
            }
        }

        public frmYahooCF()
        {
            InitializeComponent();

            cbStatus.Items.Add("Available");
            cbStatus.Items.Add("Busy");
            cbStatus.Items.Add("Stepped Out");
            cbStatus.Items.Add("Be Right Back");
            cbStatus.Items.Add("Not At Desk");
            cbStatus.Items.Add("On The Phone");
            cbStatus.Items.Add("Invisible");

            yahoo.OnError += delegate(object sender, string log) { this.Invoke(new OnLogEventHandler(OnLog), new object[] { sender, log }); };
            yahoo.OnLog += delegate(object sender, string log) { this.Invoke(new OnLogEventHandler(OnLog), new object[] { sender, log }); };
            yahoo.OnLogin += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnLogin)); };
            yahoo.OnLogout += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnLogout)); };
            yahoo.OnReceiveBuddyList += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnReceiveBuddyList)); };
            yahoo.OnReceiveGroupList += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnReceiveGroupList)); };
            yahoo.OnBuddyStatusChanged += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnBuddyStatusChanged), new object[] { sender, buddy }); };
            yahoo.OnMessage += delegate(object sender, string buddy, string message) { this.Invoke(new OnMessageEventHandler(OnMessage), new object[] { sender, buddy, message }); };
            yahoo.OnOfflineMessage += delegate(object sender, string buddy, string message, DateTime receivedDateTime) { this.Invoke(new OnOfflineMessageEventHandler(OnOfflineMessage), new object[] { sender, buddy, message, receivedDateTime }); };
            yahoo.OnStatusChanged += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnStatusChanged), new object[] { sender, e }); };
            yahoo.OnDeleteBuddy += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnDeleteBuddy), new object[] { sender, buddy }); };
            yahoo.OnAddBuddy += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnAddBuddy), new object[] { sender, buddy }); };
            yahoo.OnAddGroup += delegate(object sender, string group) { this.Invoke(new OnBuddyEventHandle(OnAddGroup), new object[] { sender, group }); };
            yahoo.OnBuddyAddYouRequest += delegate(object sender, string buddy, string requestMessage, ref bool bAccept)
            {
                object[] inVar = { sender, buddy, requestMessage, bAccept };
                this.Invoke(new OnBuddyAddYouRequestYahooEventHandler(OnBuddyAddYouRequest), inVar);
                bAccept = (bool)inVar[3];
            };
            yahoo.OnBuddyTyping += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnBuddyTyping), new object[] { sender, buddy }); };
        }

        private void OnBuddyAddYouRequest(object sender, string buddy, string requestMessage, ref bool accept)
        {
            if (MessageBox.Show("\"" + buddy + "\" added you. Do you want to \"" + buddy + "\" to see you?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question,MessageBoxDefaultButton.Button1) == DialogResult.Yes)
            {
                accept = true;
            }
            else
                accept = false;
        }

        private void OnAddGroup(object sender, string group)
        {
            TreeNode gn = new TreeNode(group);
            rootNode.Nodes.Add(gn);
            gn.SelectedImageIndex = 0;
            gn.ImageIndex = gn.SelectedImageIndex;
        }

        private void OnBuddyTyping(object sender, string buddy)
        {
            showLog(buddy + " is typing...");
        }

        private void OnAddBuddy(object sender, string buddy)
        {
            YahooBuddy yb = (YahooBuddy)yahoo.BuddyList[buddy];

            TreeNode gn = FindNode(rootNode, yb.Group);

            TreeNode bn = new TreeNode(yb.Account);
            bn.ImageIndex = statusAsImageIndex(yb.Status);
            bn.SelectedImageIndex = bn.ImageIndex;
            bn.Tag = yb;
            yb.Tag = bn;

            if (gn == null)
                rootNode.Nodes.Add(bn);
            else
                gn.Nodes.Add(bn);
        }

        private void OnDeleteBuddy(object sender, string buddy)
        {
            YahooBuddy yb = (YahooBuddy)yahoo.BuddyList[buddy];
            (yb.Tag as TreeNode).Remove();
        }

        private void OnStatusChanged(object sender, EventArgs e)
        {
            rootNode.ImageIndex = statusAsImageIndex(yahoo.Status);
            rootNode.SelectedImageIndex = rootNode.ImageIndex;

            /*
            cbStatus.Items.Add("Available");
            cbStatus.Items.Add("Busy");
            cbStatus.Items.Add("Stepped Out");
            cbStatus.Items.Add("Be Right Back");
            cbStatus.Items.Add("Not At Desk");
            cbStatus.Items.Add("On The Phone");
            cbStatus.Items.Add("Invisible");*/
            switch (yahoo.Status)
            {
                case YahooStatus.YAHOO_STATUS_AVAILABLE:
                    cbStatus.SelectedIndex = 0;
                    break;
                case YahooStatus.YAHOO_STATUS_BUSY:
                    cbStatus.SelectedIndex = 1;
                    break;
                case YahooStatus.YAHOO_STATUS_STEPPEDOUT:
                    cbStatus.SelectedIndex = 2;
                    break;
                case YahooStatus.YAHOO_STATUS_BRB:
                    cbStatus.SelectedIndex = 3;
                    break;
                case YahooStatus.YAHOO_STATUS_NOTATDESK:
                    cbStatus.SelectedIndex = 4;
                    break;
                case YahooStatus.YAHOO_STATUS_ONPHONE:
                    cbStatus.SelectedIndex = 5;
                    break;
                case YahooStatus.YAHOO_STATUS_INVISIBLE:
                    cbStatus.SelectedIndex = 6;
                    break;
            }
        }

        private int statusAsImageIndex(int status)
        {
            switch (status)
            {
                case YahooStatus.YAHOO_STATUS_AVAILABLE:
                case YahooStatus.YAHOO_STATUS_IDLE:
                    return 1;

                case YahooStatus.YAHOO_STATUS_BUSY:
                case YahooStatus.YAHOO_STATUS_CUSTOM:
                case YahooStatus.YAHOO_STATUS_ONPHONE:
                    return 3;

                case YahooStatus.YAHOO_STATUS_STEPPEDOUT:
                case YahooStatus.YAHOO_STATUS_OUTTOLUNCH:
                case YahooStatus.YAHOO_STATUS_ONVACATION:
                case YahooStatus.YAHOO_STATUS_BRB:
                case YahooStatus.YAHOO_STATUS_NOTATHOME:
                case YahooStatus.YAHOO_STATUS_NOTATDESK:
                case YahooStatus.YAHOO_STATUS_NOTINOFFICE:
                    return 2;

                default:
                    return 4;

            }
        }

        private void OnOfflineMessage(object sender, string buddy, string message, DateTime receivedDateTime)
        {
            showLog(buddy + " said(" + receivedDateTime.ToLocalTime() + "): " + message);
            sbar.Text = buddy + " said(" + receivedDateTime.ToLocalTime() + "): " + message;
        }

        private void OnMessage(object sender, string buddy, string message)
        {
            showLog(buddy + " says: " + message);
            sbar.Text = buddy + " says: " + message;
        }

        private void OnBuddyStatusChanged(object sender, string buddy)
        {
            YahooBuddy yb = (YahooBuddy)yahoo.BuddyList[buddy];
            ((TreeNode)yb.Tag).ImageIndex = statusAsImageIndex(yb.Status);
            ((TreeNode)yb.Tag).SelectedImageIndex = ((TreeNode)yb.Tag).ImageIndex;

            if ((yb.StatusMessage != null) && (yb.StatusMessage != ""))
                ((TreeNode)yb.Tag).Text = yb.Account + " - " + yb.StatusMessage;
            else
                ((TreeNode)yb.Tag).Text = yb.Account;
        }

        private void OnLogin(object sender, EventArgs e)
        {
            rootNode = new TreeNode(yahoo.Account);
            rootNode.ImageIndex = statusAsImageIndex(yahoo.Status);
            rootNode.SelectedImageIndex = rootNode.ImageIndex;

            cmdLogin.Enabled = false;
            cbStatus.Enabled = true;

            tvIM.Nodes.Add(rootNode);

            rootNode.Text = yahoo.Account;

            tabControl1.SelectedIndex = 1;

            sbar.Text = "Login";
        }

        private void OnLogout(object sender, EventArgs e)
        {
            tvIM.Nodes.Clear();

            cmdLogin.Enabled = true;
            cbStatus.Enabled = false;
            mmAction.Enabled = false;

            tabControl1.SelectedIndex = 0;

            sbar.Text = "Logout.";
        }

        private void OnLog(object sender, string log)
        {
            showLog(log);
            sbar.Text = log;
        }

        private TreeNode FindNode(TreeNode tnParent, string strValue)
        {

            if (tnParent == null) return null;

            if ((tnParent.ImageIndex == 0) && ((tnParent.Text == strValue)))
                return tnParent;

            TreeNode tnRet = null;

            foreach (TreeNode tn in tnParent.Nodes)
            {
                tnRet = FindNode(tn, strValue);
                if (tnRet != null) break;
            }
            return tnRet;
        }

        private void OnReceiveBuddyList(object sender, EventArgs e)
        {
            foreach (DictionaryEntry buddy in yahoo.BuddyList)
            {
                YahooBuddy yb = (YahooBuddy)buddy.Value;

                TreeNode gn = FindNode(rootNode, yb.Group);

                TreeNode bn = new TreeNode(yb.Account);
                bn.ImageIndex = statusAsImageIndex(yb.Status);
                bn.SelectedImageIndex = bn.ImageIndex;
                bn.Tag = yb;
                yb.Tag = bn;

                if (gn == null)
                    rootNode.Nodes.Add(bn);
                else
                    gn.Nodes.Add(bn);
            }
            rootNode.ExpandAll();
        }

        private void OnReceiveGroupList(object sender, EventArgs e)
        {
            foreach (DictionaryEntry group in yahoo.GroupList)
            {
                string g = (string)group.Value;

                TreeNode gn = new TreeNode(g);
                rootNode.Nodes.Add(gn);
                gn.ImageIndex = 0;
                gn.SelectedImageIndex = gn.ImageIndex;
            }

            rootNode.ExpandAll();
        }



        private void Form1_Load(object sender, EventArgs e)
        {

        }

        private void cmdLogin_Click(object sender, EventArgs e)
        {
            showLog("Start Login...");
            if (cbInvisible.Checked)
                yahoo.Status = YahooStatus.YAHOO_STATUS_INVISIBLE;

            yahoo.Account = txtEmail.Text;
            yahoo.Password = txtPassword.Text;
            yahoo.Login();
        }

        private void sbar_ParentChanged(object sender, EventArgs e)
        {

        }

        private void menuItem3_Click(object sender, EventArgs e)
        {
            cmdLogin_Click(sender, e);
        }

        private void menuItem4_Click(object sender, EventArgs e)
        {
            yahoo.Logout();
        }

        private void menuItem14_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void menuItem2_Click(object sender, EventArgs e)
        {
            if ((tvIM.SelectedNode != null) && (tvIM.SelectedNode.Tag is YahooBuddy))
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.SetCaption("Instant Message:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                {
                    yahoo.SendMessage((tvIM.SelectedNode.Tag as YahooBuddy).Account, frmInput.MessageStr);
                }
            }
        }

        private void menuItem5_Click(object sender, EventArgs e)
        {
            if ((tvIM.SelectedNode != null) && (tvIM.SelectedNode.Tag is YahooBuddy))
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.SetCaption("Offline Message:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                {
                    yahoo.SendOfflineMessage((tvIM.SelectedNode.Tag as YahooBuddy).Account, frmInput.MessageStr);
                }
            }
        }

        private void menuItem6_Click(object sender, EventArgs e)
        {
            if ((tvIM.SelectedNode != null) && (tvIM.SelectedNode.Tag is YahooBuddy))
                yahoo.SendTypingMessage((tvIM.SelectedNode.Tag as YahooBuddy).Account);
        }

        private void menuItem10_Click(object sender, EventArgs e)
        {
            if ((tvIM.SelectedNode != null) && (tvIM.SelectedNode.Tag is YahooBuddy))
            {
                if (MessageBox.Show("Are you sure to you want to delete \"" + (tvIM.SelectedNode.Tag as YahooBuddy).Account + "\".", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1) == DialogResult.Yes)
                    yahoo.DeleteBuddy(tvIM.SelectedNode.Tag as YahooBuddy);
            }
        }

        private void menuItem9_Click(object sender, EventArgs e)
        {
            //add buddy
            if (yahoo.Logined)
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.SetCaption("Add Buddy:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                    yahoo.AddBuddy("Friends", frmInput.MessageStr, "Bricksoft Yahoo Robot");
            }
        }

        private void cbStatus_SelectedIndexChanged(object sender, EventArgs e)
        {
            /*
            cbStatus.Items.Add("Available");
            cbStatus.Items.Add("Busy");
            cbStatus.Items.Add("Stepped Out");
            cbStatus.Items.Add("Be Right Back");
            cbStatus.Items.Add("Not At Desk");
            cbStatus.Items.Add("On The Phone");
            cbStatus.Items.Add("Invisible");*/
            switch (cbStatus.SelectedIndex)
            {
                case 0:
                    yahoo.ChangeStatus(YahooStatus.YAHOO_STATUS_AVAILABLE, "");
                    break;
                case 1:
                case 5:
                    yahoo.ChangeStatus(YahooStatus.YAHOO_STATUS_BUSY, "");
                    break;
                case 6:
                    yahoo.ChangeStatus(YahooStatus.YAHOO_STATUS_INVISIBLE, "");
                    break;
                default:
                    yahoo.ChangeStatus(YahooStatus.YAHOO_STATUS_BRB, "");
                    break;
            }

        }
    }
}